﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using ThinkGeo.MapSuite.WebEdition;
using ThinkGeo.MapSuite.Core;
using System.Configuration;
using System.Collections.ObjectModel;
using System.Data;

namespace WebApplication2
{
    public partial class _Default : System.Web.UI.Page
    {
        static DataTable dataTable = null;
        static string featureReadyToChangeId = "";
        protected void Page_Load(object sender, EventArgs e)
        {
            if (!Page.IsPostBack)
            {
                Map1.MapBackground.BackgroundBrush = new GeoSolidBrush(GeoColor.FromHtml("#E5E3DF"));
                Map1.CurrentExtent = new RectangleShape(-13939426.6371, 6701997.4056, -7812401.86, 2626987.386962);
                Map1.MapUnit = GeographyUnit.Meter;

                Map1.MapTools.OverlaySwitcher.Enabled = true;
                Map1.MapTools.MouseCoordinate.Enabled = true;

                GoogleOverlay google = new GoogleOverlay("Google Map");
                google.JavaScriptLibraryUri = new Uri(ConfigurationManager.AppSettings["GoogleUri"]);
                google.GoogleMapType = GoogleMapType.Normal;

                ShapeFileFeatureLayer shapeFileFeatureLayer = new ShapeFileFeatureLayer(MapPath("~/SampleData/USA/cities_c2.shp"), ShapeFileReadWriteMode.ReadWrite);
                shapeFileFeatureLayer.ZoomLevelSet.ZoomLevel01.DefaultPointStyle = PointStyles.CreateSimplePointStyle(PointSymbolType.Circle, GeoColor.FromArgb(100, 212, 220, 184), GeoColor.FromArgb(255, 132, 132, 154), 6);
                shapeFileFeatureLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;
                Proj4Projection proj4 = new Proj4Projection();
                proj4.InternalProjectionParametersString = Proj4Projection.GetEpsgParametersString(4326);
                proj4.ExternalProjectionParametersString = Proj4Projection.GetGoogleMapParametersString();
                shapeFileFeatureLayer.FeatureSource.Projection = proj4;

                LayerOverlay shapeOverlay = new LayerOverlay("Shape Overlay", false, TileType.SingleTile);
                shapeOverlay.Layers.Add("WorldLayer", shapeFileFeatureLayer);
                shapeOverlay.TransitionEffect = TransitionEffect.None;

                Map1.CustomOverlays.Add(google);
                Map1.CustomOverlays.Add(shapeOverlay);
            }
        }

        protected void Map1_Click(object sender, MapClickedEventArgs e)
        {
            LayerOverlay staticOverlay = (LayerOverlay)Map1.CustomOverlays["Shape Overlay"];
            FeatureLayer worldLayer = (FeatureLayer)staticOverlay.Layers["WorldLayer"];
            worldLayer.Open();
            Collection<Feature> selectedFeatures = worldLayer.QueryTools.GetFeaturesNearestTo(e.Position, Map1.MapUnit, 1, new string[2] { "AREANAME", "POP2000"});
            worldLayer.Close();

            if (selectedFeatures.Count > 0)
            {
                FeaturesGridView.DataSource = GetDataTableFromFeatureSourceColumns(selectedFeatures);
                FeaturesGridView.DataBind();
            }
        }

        private static DataTable GetDataTableFromFeatureSourceColumns(Collection<Feature> features)
        {
            dataTable = new DataTable();

            dataTable.Columns.Add("AREANAME");
            dataTable.Columns.Add("POP2000");

            foreach (Feature feature in features)
            {
                AddRow(feature, dataTable);
            }

            return dataTable;
        }

        private static void AddRow(Feature feature, DataTable dataTable)
        {
            DataRow dataRow = dataTable.NewRow();

            dataRow[0] = feature.ColumnValues["AREANAME"];
            dataRow[1] = feature.ColumnValues["POP2000"];
            dataTable.Rows.Add(dataRow);
        }

        protected void FeaturesGridView_RowEditing(object sender, System.Web.UI.WebControls.GridViewEditEventArgs e)
        {
            FeaturesGridView.EditIndex = e.NewEditIndex;
            FeaturesGridView.DataSource = dataTable;
            FeaturesGridView.DataBind();
        }

        protected void FeaturesGridView_RowCancelingEdit(object sender, GridViewCancelEditEventArgs e)
        {
            FeaturesGridView.EditIndex = -1;
            FeaturesGridView.DataSource = dataTable;
            FeaturesGridView.DataBind();
        }

        protected void FeaturesGridView_RowUpdating(object sender, GridViewUpdateEventArgs e)
        {
            DataTable dt = dataTable;

            //Update the grid values
            GridViewRow row = FeaturesGridView.Rows[e.RowIndex];
            dt.Rows[row.DataItemIndex]["AREANAME"] = ((TextBox)(row.Cells[1].Controls[0])).Text;
            dt.Rows[row.DataItemIndex]["POP2000"] = ((TextBox)(row.Cells[2].Controls[0])).Text;

            //Update the shapefile values.
            LayerOverlay staticOverlay = (LayerOverlay)Map1.CustomOverlays["Shape Overlay"];
            FeatureLayer worldLayer = (FeatureLayer)staticOverlay.Layers["WorldLayer"];
            worldLayer.Open();
            worldLayer.EditTools.BeginTransaction();
            Feature currentFeature = worldLayer.FeatureSource.GetFeatureById(featureReadyToChangeId, ReturningColumnsType.AllColumns);
            Dictionary<string, string> columnValues = currentFeature.ColumnValues;
            columnValues["AREANAME"] = ((TextBox)(row.Cells[1].Controls[0])).Text;
            columnValues["POP2000"] = ((TextBox)(row.Cells[2].Controls[0])).Text;
            Feature feature = new Feature(currentFeature.GetWellKnownBinary(), currentFeature.Id, columnValues);
            worldLayer.EditTools.Update(feature);
            worldLayer.EditTools.CommitTransaction();
            worldLayer.Close();

            //Reset the edit index.
            FeaturesGridView.EditIndex = -1;
            FeaturesGridView.DataSource = dataTable;
            FeaturesGridView.DataBind();
        }
    }
}
